<?php

namespace JimsCustomerBooking\Integrations;

use JimsCustomerBooking\Helpers\Http;

/**
 * Class SlackNotification
 */
class SlackNotification extends ANotification
{
    /**
     * @var string $service
     */
    protected string $service = 'slack';

    /**
     * @var array $messages
     */
    protected array $messages = [];


    /**
     * @var string $webhook
     */
    protected string $webhook = '';

    public function __construct(string $webhook, $vars = [], $test_mode = false)
    {
        $this->webhook = $webhook;
        $this->setupNotificationMessages ($vars);
        $this->test_mode = $test_mode;
    }


    /**
     * @param string $type
     * @return bool
     */
    public function send(string $type = self::NOTIFICATION_TYPE_NOTIFICATION)
    {
        //send notification to slack
        if ($this->test_mode) return $this->getMessage ($type);

        $this->setServiceResponse (Http::Post ($this->webhook, $this->getMessage ($type)));

        //check if slack responded with ok
        if (is_array ($this->getServiceResponse ()) && !empty($this->getServiceResponse ()) && $this->hasResponseProperty ()) {
            return $this->isOK () ?? false;
        }

        return false;
    }

    /**
     * @param $vars
     * @return void
     */
    public function setupNotificationMessages($vars = [])
    {
        $notification = __('*Booking request received*. For more info check leads site for booking: {{booking_id}}');
        $info = __('*Booking request received*. For more info check leads site for booking: {{booking_id}}');
        $error = __('*Booking request failed*. For more info check leads site for booking: {{error_id}}');


        //build block messages for slack
        $this->messages = [
        self::NOTIFICATION_TYPE_NOTIFICATION => [
                'text' => $this->replaceVars ($notification, $vars),
                'blocks' => [
                    [
                        'type' => 'section',
                        'text' => [
                            'type' => 'mrkdwn',
                            'text' => $this->replaceVars ($notification, $vars)
                        ]
                    ]
                ]
            ],
            self::NOTIFICATION_TYPE_INFO => [
                'text' => $this->replaceVars ( $info, $vars),
                'blocks' => [
                    [
                        'type' => 'section',
                        'text' => [
                            'type' => 'mrkdwn',
                            'text' => $this->replaceVars ($info, $vars)
                        ]
                    ]
                ]
        ],
            self::NOTIFICATION_TYPE_ERROR => [
                'text' => $this->replaceVars ($error, $vars),
                'blocks' => [
                    [
                        'type' => 'section',
                        'text' => [
                            'type' => 'mrkdwn',
                            'text' => $this->replaceVars ($error, $vars)
                        ]
                    ]
                ]
            ]
        ];
    }

    /**
     * @param $message
     * @param array $vars
     * @return string|string[]
     */
    protected function replaceVars($message, $vars = []): array|string
    {
        //replace vars in messages
        //for each var in vars, replace the key with the value in the message
        foreach ($vars as $key => $value) {
            $message = str_replace ('{{' . $key . '}}', $value, $message);
        }

        return $message;
    }

    /**
     * @return bool
     */
    protected function hasResponseProperty(): bool
    {
        return isset($this->getServiceResponse ()['result']) && !empty($this->getServiceResponse ()['result']);
    }

    /**
     * @return mixed
     */
    protected function isOK(): mixed
    {
        return $this->getServiceResponse ()['result'] == 'ok';
    }
}